#pragma once

#include <string>
using namespace std;

// Forward declaration of the RefCountedStringRep class.
class RefCountedStringRep;


class RefCountedString
{
// State.
private:
	RefCountedStringRep * pRep;			// Pointer to a shareable representation object.


// Constructors, assignment, and destructor.
public: 
	RefCountedString(const char * text = "");
	RefCountedString(const RefCountedString & other);
	RefCountedString & operator = (const RefCountedString & other);

	~RefCountedString();


// Modifier methods/operators. 
public: 
	RefCountedString & ToUpperCase();
	RefCountedString & ToLowerCase();
	RefCountedString & Append(const RefCountedString & other);
	RefCountedString & operator +=(const RefCountedString & other);
	RefCountedString & SetAt(int index, char ch);
	istream & InputFromStream(istream & is);


// Const methods/operators.
public: 
	char GetAt(int index) const;
	bool operator!() const;
	bool Equals(const RefCountedString & other) const;
	ostream & OutputToStream(ostream & os) const;

// Private methods, to manage the shareable representation object.
private:
	void HookExistingRep(RefCountedStringRep * pRep);
	void UnhookExistingRep();
	void EnsureUniqueRep();

};


// Global binary operators (where the 1st parameter might not be a RefCountedString).
bool operator==(const RefCountedString & lhs, const RefCountedString & rhs);
bool operator!=(const RefCountedString & lhs, const RefCountedString & rhs);

istream & operator >> (istream & is, RefCountedString & str);
ostream & operator << (ostream & os, const RefCountedString & str);

